# [jQuery asScroll](https://github.com/amazingSurge/jquery-asScroll) ![bower][bower-image] [![NPM version][npm-image]][npm-url] [![Dependency Status][daviddm-image]][daviddm-url] [![prs-welcome]](#contributing)

> A jquery plugin that click on the button to jump to the specified location.

## Table of contents
- [Main files](#main-files)
- [Quick start](#quick-start)
- [Requirements](#requirements)
- [Usage](#usage)
- [Examples](#examples)
- [Options](#options)
- [Methods](#methods)
- [Events](#events)
- [No conflict](#no-conflict)
- [Browser support](#browser-support)
- [Contributing](#contributing)
- [Development](#development)
- [Changelog](#changelog)
- [Copyright and license](#copyright-and-license)

## Main files
```
dist/
├── jquery-asScroll.js
├── jquery-asScroll.es.js
└── jquery-asScroll.min.js
```

## Quick start
Several quick start options are available:
#### Download the latest build

 * [Development](https://raw.githubusercontent.com/amazingSurge/jquery-asScroll/master/dist/jquery-asScroll.js) - unminified
 * [Production](https://raw.githubusercontent.com/amazingSurge/jquery-asScroll/master/dist/jquery-asScroll.min.js) - minified

#### Install From Bower
```sh
bower install jquery-asScroll --save
```

#### Install From Npm
```sh
npm install jquery-asScroll --save
```

#### Build From Source
If you want build from source:

```sh
git clone git@github.com:amazingSurge/jquery-asScroll.git
cd jquery-asScroll
npm install
npm install -g gulp-cli babel-cli
gulp build
```

Done!

## Requirements
`jquery-asScroll` requires the latest version of [`jQuery`](https://jquery.com/download/).

## Usage
#### Including files:

```html
<script src="/path/to/jquery.js"></script>
<script src="/path/to/jquery-asScroll.js"></script>
```

#### Required HTML structure

```html
<div data-scroll-target="basic">
  Content
</div>
```

#### Initialization
All you need to do is call the plugin on the element:

```javascript
jQuery(function($) {
  $("body").asScroll();
});
```

## Examples
There are some example usages that you can look at to get started. They can be found in the
[examples folder](https://github.com/amazingSurge/jquery-asScroll/tree/master/examples).

## Options
`jquery-asScroll` can accept an options object to alter the way it behaves. You can see the default options by call `$.asScroll.setDefaults()`. The structure of an options object is as follows:

```
{
  duration: 800,
  easing: 'ease',
  namespace: 'asScroll',
  offsetTop: 50,
  mobile: {
    width: 768,
    duration: 500,
    easing: 'ease',
  }
}
```

## Methods
Methods are called on asScroll instances through the asScroll method itself.
You can also save the instances to variable for further use.

```javascript
// call directly
$().asScroll('method');

// or
var api = $().data('asScroll');
api.method();
```

## Events
`jquery-asScroll` provides custom events for the plugin’s unique actions. 

```javascript
$(document).on('active', function (e, direction, target) {
  // on scroll target is active
});

```

Event  | Description
------ | -----------
active | Fires when the scroll target element is active on scrolling.

## No conflict
If you have to use other plugin with the same namespace, just call the `$.asScroll.noConflict` method to revert to it.

```html
<script src="other-plugin.js"></script>
<script src="jquery-asScroll.js"></script>
<script>
  $.asScroll.noConflict();
  // Code that uses other plugin's "$().asScroll" can follow here.
</script>
```

## Browser support

Tested on all major browsers.

| <img src="https://raw.githubusercontent.com/alrra/browser-logos/master/safari/safari_32x32.png" alt="Safari"> | <img src="https://raw.githubusercontent.com/alrra/browser-logos/master/chrome/chrome_32x32.png" alt="Chrome"> | <img src="https://raw.githubusercontent.com/alrra/browser-logos/master/firefox/firefox_32x32.png" alt="Firefox"> | <img src="https://raw.githubusercontent.com/alrra/browser-logos/master/edge/edge_32x32.png" alt="Edge"> | <img src="https://raw.githubusercontent.com/alrra/browser-logos/master/internet-explorer/internet-explorer_32x32.png" alt="IE"> | <img src="https://raw.githubusercontent.com/alrra/browser-logos/master/opera/opera_32x32.png" alt="Opera"> |
|:--:|:--:|:--:|:--:|:--:|:--:|
| Latest ✓ | Latest ✓ | Latest ✓ | Latest ✓ | 9-11 ✓ | Latest ✓ |

As a jQuery plugin, you also need to see the [jQuery Browser Support](http://jquery.com/browser-support/).

## Contributing
Anyone and everyone is welcome to contribute. Please take a moment to
review the [guidelines for contributing](CONTRIBUTING.md). Make sure you're using the latest version of `jquery-asScroll` before submitting an issue. There are several ways to help out:

* [Bug reports](CONTRIBUTING.md#bug-reports)
* [Feature requests](CONTRIBUTING.md#feature-requests)
* [Pull requests](CONTRIBUTING.md#pull-requests)
* Write test cases for open bug issues
* Contribute to the documentation

## Development
`jquery-asScroll` is built modularly and uses Gulp as a build system to build its distributable files. To install the necessary dependencies for the build system, please run:

```sh
npm install -g gulp
npm install -g babel-cli
npm install
```

Then you can generate new distributable files from the sources, using:
```
gulp build
```

More gulp tasks can be found [here](CONTRIBUTING.md#available-tasks).

## Changelog
To see the list of recent changes, see [Releases section](https://github.com/amazingSurge/jquery-asScroll/releases).

## Copyright and license
Copyright (C) 2016 amazingSurge.

Licensed under [the LGPL license](LICENSE).

[⬆ back to top](#table-of-contents)

[bower-image]: https://img.shields.io/bower/v/jquery-asScroll.svg?style=flat
[bower-link]: https://david-dm.org/amazingSurge/jquery-asScroll/dev-status.svg
[npm-image]: https://badge.fury.io/js/jquery-asScroll.svg?style=flat
[npm-url]: https://npmjs.org/package/jquery-asScroll
[license]: https://img.shields.io/npm/l/jquery-asScroll.svg?style=flat
[prs-welcome]: https://img.shields.io/badge/PRs-welcome-brightgreen.svg
[daviddm-image]: https://david-dm.org/amazingSurge/jquery-asScroll.svg?style=flat
[daviddm-url]: https://david-dm.org/amazingSurge/jquery-asScroll
