/**
* jQuery asScroll v0.1.1
* https://github.com/amazingSurge/jquery-asScroll
*
* Copyright (c) amazingSurge
* Released under the LGPL-3.0 license
*/
(function(global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['jquery'], factory);
  } else if (typeof exports !== "undefined") {
    factory(require('jquery'));
  } else {
    var mod = {
      exports: {}
    };
    factory(global.jQuery);
    global.jqueryAsScrollEs = mod.exports;
  }
})(this,

  function(_jquery) {
    'use strict';

    var _jquery2 = _interopRequireDefault(_jquery);

    function _interopRequireDefault(obj) {
      return obj && obj.__esModule ? obj : {
        default: obj
      };
    }

    var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ?

      function(obj) {
        return typeof obj;
      }
      :

      function(obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
      };

    function _classCallCheck(instance, Constructor) {
      if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
      }
    }

    var _createClass = function() {
      function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
          var descriptor = props[i];
          descriptor.enumerable = descriptor.enumerable || false;
          descriptor.configurable = true;

          if ("value" in descriptor)
            descriptor.writable = true;
          Object.defineProperty(target, descriptor.key, descriptor);
        }
      }

      return function(Constructor, protoProps, staticProps) {
        if (protoProps)
          defineProperties(Constructor.prototype, protoProps);

        if (staticProps)
          defineProperties(Constructor, staticProps);

        return Constructor;
      };
    }();

    if (!Date.now) {
      Date.now = function() {
        return new Date().getTime();
      }
      ;
    }

    var vendors = ['webkit', 'moz'];

    for (var i = 0; i < vendors.length && !window.requestAnimationFrame; ++i) {
      var vp = vendors[i];
      window.requestAnimationFrame = window[vp + 'RequestAnimationFrame'];
      window.cancelAnimationFrame = window[vp + 'CancelAnimationFrame'] || window[vp + 'CancelRequestAnimationFrame'];
    }

    if (/iP(ad|hone|od).*OS (6|7|8)/.test(window.navigator.userAgent) || !window.requestAnimationFrame || !window.cancelAnimationFrame) {
      (function() {
        var lastTime = 0;
        window.requestAnimationFrame = function(callback) {
          var now = getTime();
          var timePlus = 16;
          var nextTime = Math.max(lastTime + timePlus, now);

          return setTimeout(

            function() {
              callback(lastTime = nextTime);
            }
            , nextTime - now);
        }
        ;
        window.cancelAnimationFrame = clearTimeout;
      })();
    }

    /**
     * Helper functions
     **/

    function isPercentage(n) {
      'use strict';

      return typeof n === 'string' && n.indexOf('%') !== -1;
    }

    function convertPercentageToFloat(n) {
      'use strict';

      return parseFloat(n.slice(0, -1) / 100, 10);
    }

    function getTime() {
      if (typeof window.performance !== 'undefined' && window.performance.now) {

        return window.performance.now();
      }

      return Date.now();
    }

    var DEFAULTS = {
      duration: 800,
      easing: 'ease',
      namespace: 'asScroll',
      offsetTop: 50,
      mobile: {
        width: 768,
        duration: 500,
        easing: 'ease'
      }
    };

    var easingBezier = function easingBezier(mX1, mY1, mX2, mY2) {
      'use strict';

      var a = function a(aA1, aA2) {
        return 1.0 - 3.0 * aA2 + 3.0 * aA1;
      };

      var b = function b(aA1, aA2) {
        return 3.0 * aA2 - 6.0 * aA1;
      };

      var c = function c(aA1) {
        return 3.0 * aA1;
      };

      // Returns x(t) given t, x1, and x2, or y(t) given t, y1, and y2.
      var calcBezier = function calcBezier(aT, aA1, aA2) {
        return ((a(aA1, aA2) * aT + b(aA1, aA2)) * aT + c(aA1)) * aT;
      };

      // Returns dx/dt given t, x1, and x2, or dy/dt given t, y1, and y2.
      var getSlope = function getSlope(aT, aA1, aA2) {
        return 3.0 * a(aA1, aA2) * aT * aT + 2.0 * b(aA1, aA2) * aT + c(aA1);
      };

      var getTForX = function getTForX(aX) {
        // Newton raphson iteration
        var aGuessT = aX;

        for (var _i = 0; _i < 4; ++_i) {
          var currentSlope = getSlope(aGuessT, mX1, mX2);

          if (currentSlope === 0.0) {

            return aGuessT;
          }
          var currentX = calcBezier(aGuessT, mX1, mX2) - aX;
          aGuessT -= currentX / currentSlope;
        }

        return aGuessT;
      };

      if (mX1 === mY1 && mX2 === mY2) {

        return {
          css: 'linear',
          fn: function fn(aX) {
            return aX;
          }
        };
      }

      return {
        css: 'cubic-bezier(' + mX1 + ',' + mY1 + ',' + mX2 + ',' + mY2 + ')',
        fn: function fn(aX) {
          return calcBezier(getTForX(aX), mY1, mY2);
        }
      };
    };

    var EASING = {
      ease: easingBezier(0.25, 0.1, 0.25, 1.0),
      linear: easingBezier(0.00, 0.0, 1.00, 1.0),
      'ease-in': easingBezier(0.42, 0.0, 1.00, 1.0),
      'ease-out': easingBezier(0.00, 0.0, 0.58, 1.0),
      'ease-in-out': easingBezier(0.42, 0.0, 0.58, 1.0)
    };

    var NAMESPACE$1 = 'AsScroll';

    var instanceId = 0;

    var requestAnimFrame = function() {
      return window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame ||

        function(callback) {
          window.setTimeout(callback, 1000 / 60);
        };
    }();

    /**
     * Plugin constructor
     **/

    var AsScroll = function() {
      function AsScroll(element, options) {
        _classCallCheck(this, AsScroll);

        this.element = element;
        this.$element = (0, _jquery2.default)(element);
        this.options = _jquery2.default.extend(true, {}, DEFAULTS, options);

        if (this.options.containerSelector) {
          this.$container = (0, _jquery2.default)(this.options.containerSelector);
        } else {
          this.$container = this.$element.is(document.body) ? (0, _jquery2.default)(window) : this.$element.parent();
        }

        if (this.$container.length !== 1) {

          return;
        }

        this.namespace = this.options.namespace;
        this.attributes = {
          vertical: {
            axis: 'Y',
            overflow: 'overflow-y',

            scroll: 'scrollTop',
            scrollLength: 'scrollHeight',
            pageOffset: 'pageYOffset',

            ffPadding: 'padding-right',

            length: 'height',
            clientLength: 'clientHeight',
            offsetLength: 'offsetHeight',
            offset: 'top',

            crossOffset: 'left',
            crossLength: 'width',
            crossClientLength: 'clientWidth',
            crossOffsetLength: 'offsetWidth'
          },
          horizontal: {
            axis: 'X',
            overflow: 'overflow-x',

            scroll: 'scrollLeft',
            scrollLength: 'scrollWidth',
            pageOffset: 'pageXOffset',

            ffPadding: 'padding-bottom',

            length: 'width',
            clientLength: 'clientWidth',
            offsetLength: 'offsetWidth',
            offset: 'left',

            crossOffset: 'top',
            crossLength: 'height',
            crossClientLength: 'clientHeight',
            crossOffsetLength: 'offsetHeight'
          }
        };

        this.classes = {};
        this.easing = EASING[this.options.easing] || EASING.ease;
        this.duration = this.options.duration;

        this._frameId = null;
        this._states = {};
        this.instanceId = ++instanceId;

        this.vertical = false;
        this.horizontal = false;

        this.init();
      }

      _createClass(AsScroll, [{
        key: 'init',
        value: function init() {
          var that = this;
          this.$targets = this.$element.find('[data-scroll-target]');
          this.lists = {};

          if (this.getScrollLength('vertical') > 0) {
            this.vertical = true;
          }

          if (this.getScrollLength('horizontal') > 0) {
            this.horizontal = true;
          }

          this.$targets.each(

            function() {
              var $target = (0, _jquery2.default)(this);

              that.lists[$target.data('scrollTarget')] = {
                $el: $target,
                offset: $target.offset()
              };
            }
          );

          this.bindEvents();
        }
      }, {
        key: 'getActiveTarget',
        value: function getActiveTarget(direction) {
          /* eslint consistent-return: "off" */

          if (!this[direction]) {

            return;
          }

          var offset = this.getOffset(direction);
          var attributes = this.attributes[direction];
          var activeTarget = null;
          var containerLength = this.getContainerLength(direction);

          _jquery2.default.each(this.lists,

            function(target, obj) {
              var targetOffset = obj.offset[attributes.offset];
              var length = obj.$el[attributes.length]();

              if (targetOffset === offset) {
                activeTarget = target;

                return false;
              } else if (targetOffset > offset && targetOffset < offset + containerLength) {
                activeTarget = target;

                return false;
              } else if (targetOffset < offset && targetOffset + length > offset) {
                activeTarget = target;

                return true;
              } else if (targetOffset > offset + containerLength) {

                return false;
              }
            }
          );

          return activeTarget;
        }
      }, {
        key: 'bindEvents',
        value: function bindEvents() {
          var that = this;

          (0, _jquery2.default)(window).on(this.eventNameWithId('resize'),

            function() {
              if ((0, _jquery2.default)(window).width() < that.options.mobile.width) {
                that.duration = that.options.duration;
                that.easing = AsScroll.easing[that.options.easing] || AsScroll.easing.ease;
              }
            }
          );

          (0, _jquery2.default)(window).on(this.eventNameWithId('orientationchange'),

            function() {
              if ((0, _jquery2.default)(window).width() < that.options.mobile.width) {
                that.duration = that.options.duration;
                that.easing = AsScroll.easing[that.options.easing] || AsScroll.easing.ease;
              }
            }
          );

          if (!this.horizontal && !this.vertical) {

            return;
          }

          this.$container.on(this.eventName('scroll'),

            function() {
              if (that.vertical) {
                that.trigger(that.eventName('active'), ['vertical', that.getActiveTarget('vertical')]);
              }

              if (that.horizontal) {
                that.getActiveTarget('horizontal');
                that.trigger(that.eventName('active'), ['horizontal', that.getActiveTarget('horizontal')]);
              }
            }
          );
        }
      }, {
        key: 'eventName',
        value: function eventName(events) {
          if (typeof events !== 'string' || events === '') {

            return '.' + this.options.namespace;
          }

          events = events.split(' ');
          var length = events.length;

          for (var _i2 = 0; _i2 < length; _i2++) {
            events[_i2] = events[_i2] + '.' + this.options.namespace;
          }

          return events.join(' ');
        }
      }, {
        key: 'eventNameWithId',
        value: function eventNameWithId(events) {
          if (typeof events !== 'string' || events === '') {

            return this.options.namespace + '-' + this.instanceId;
          }

          events = events.split(' ');
          var length = events.length;

          for (var _i3 = 0; _i3 < length; _i3++) {
            events[_i3] = events[_i3] + '.' + this.options.namespace + '-' + this.instanceId;
          }

          return events.join(' ');
        }
      }, {
        key: 'trigger',
        value: function trigger(eventType) {
          var _ref;

          for (var _len = arguments.length, params = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            params[_key - 1] = arguments[_key];
          }

          var data = (_ref = [this]).concat.apply(_ref, params);

          // event
          this.$element.trigger(NAMESPACE$1 + '::' + eventType, data);

          // callback
          eventType = eventType.replace(/\b\w+\b/g,

            function(word) {
              return word.substring(0, 1).toUpperCase() + word.substring(1);
            }
          );
          var onFunction = 'on' + eventType;

          if (typeof this.options[onFunction] === 'function') {
            var _options$onFunction;

            (_options$onFunction = this.options[onFunction]).apply.apply(_options$onFunction, [this].concat(params));
          }
        }
      }, {
        key: 'is',
        value: function is(state) {
          return this._states[state] && this._states[state] > 0;
        }
      }, {
        key: 'enter',
        value: function enter(state) {
          if (this._states[state] === undefined) {
            this._states[state] = 0;
          }

          this._states[state]++;
        }
      }, {
        key: 'leave',
        value: function leave(state) {
          this._states[state]--;
        }
      }, {
        key: 'getOffset',
        value: function getOffset(direction) {
          var attributes = this.attributes[direction];
          var element = this.$element[0];

          return element[attributes.pageOffset] || element[attributes.scroll];
        }
      }, {
        key: 'getPercentOffset',
        value: function getPercentOffset(direction) {
          return this.getOffset(direction) / this.getScrollLength(direction);
        }
      }, {
        key: 'getContainerLength',
        value: function getContainerLength(direction) {
          return this.$container[0] === window ? this.$container[this.attributes[direction].length]() : this.$container[0][this.attributes[direction].clientLength];
        }
      }, {
        key: 'getScrollLength',
        value: function getScrollLength(direction) {
          var scrollLength = this.$element[0][this.attributes[direction].scrollLength];

          return scrollLength - this.getContainerLength(direction);
        }
      }, {
        key: 'scrollToTarget',
        value: function scrollToTarget(direction, target, trigger, sync) {
          if (typeof this.lists[target] === 'undefined') {

            return;
          }
          target = this.lists[target];

          var attributes = this.attributes[direction];
          var offset = target.offset[attributes.offset];

          this.scrollTo(direction, offset, trigger, sync);
        }
      }, {
        key: 'scrollTo',
        value: function scrollTo(direction, value, trigger, sync) {
          var type = typeof value === 'undefined' ? 'undefined' : _typeof(value);

          if (type === "string") {

            if (isPercentage(value)) {
              value = convertPercentageToFloat(value) * this.getScrollLength(direction);
            }

            value = parseFloat(value);
            type = "number";
          }

          if (type !== "number") {

            return;
          }
          this.move(direction, value, trigger, sync);
        }
      }, {
        key: 'scrollBy',
        value: function scrollBy(direction, value, trigger, sync) {
          var type = typeof value === 'undefined' ? 'undefined' : _typeof(value);

          if (type === "string") {

            if (isPercentage(value)) {
              value = convertPercentageToFloat(value) * this.getScrollLength(direction);
            }

            value = parseFloat(value);
            type = "number";
          }

          if (type !== "number") {

            return;
          }

          this.move(direction, this.getOffset(direction) + value, trigger, sync);
        }
      }, {
        key: 'move',
        value: function move(direction, value, trigger, sync) {
          if (!this[direction] || typeof value !== "number") {

            return;
          }

          var that = this;

          this.enter('moving');

          if (value < 0) {
            value = 0;
          } else if (value > this.getScrollLength(direction)) {
            value = this.getScrollLength(direction);
          }

          var attributes = this.attributes[direction];

          var callback = function callback() {
            that.leave('moving');
          };

          if (sync) {

            this.$element[0][attributes.scroll] = value;

            if (trigger !== false) {
              this.trigger('change', value / this.getScrollLength(direction));
            }
            callback();
          } else {
            (function() {
              that.enter('animating');
              var startTime = getTime();
              var start = that.getOffset(direction);
              var end = value;

              var run = function run(time) {
                var percent = (time - startTime) / that.duration;

                if (percent > 1) {
                  percent = 1;
                }

                percent = that.easing.fn(percent);

                var current = parseFloat(start + percent * (end - start), 10);

                that.$element[0][attributes.scroll] = current;

                if (trigger !== false) {
                  that.trigger('change', value / that.getScrollLength(direction));
                }

                if (percent === 1) {
                  cancelAnimationFrame(that._frameId);
                  that._frameId = null;

                  that.leave('animating');
                  callback();
                } else {
                  that._frameId = requestAnimFrame(run);
                }
              };

              that._frameId = requestAnimFrame(run);
            })();
          }
        }
      }, {
        key: 'scrollXto',
        value: function scrollXto(value, trigger, sync) {
          return this.scrollTo('horizontal', value, trigger, sync);
        }
      }, {
        key: 'scrollYto',
        value: function scrollYto(value, trigger, sync) {
          return this.scrollTo('vertical', value, trigger, sync);
        }
      }, {
        key: 'scrollXby',
        value: function scrollXby(value, trigger, sync) {
          return this.scrollBy('horizontal', value, trigger, sync);
        }
      }, {
        key: 'scrollYby',
        value: function scrollYby(value, trigger, sync) {
          return this.scrollBy('vertical', value, trigger, sync);
        }
      }, {
        key: 'scrollXToTarget',
        value: function scrollXToTarget(target, trigger, sync) {
          return this.scrollToTarget('horizontal', target, trigger, sync);
        }
      }, {
        key: 'scrollYToTarget',
        value: function scrollYToTarget(target, trigger, sync) {
          return this.scrollToTarget('vertical', target, trigger, sync);
        }
      }], [{
        key: 'registerEasing',
        value: function registerEasing(name) {
          for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            args[_key2 - 1] = arguments[_key2];
          }

          EASING[name] = easingBezier.apply(undefined, args);
        }
      }, {
        key: 'getEasing',
        value: function getEasing(name) {
          return EASING[name];
        }
      }, {
        key: 'setDefaults',
        value: function setDefaults(options) {
          _jquery2.default.extend(true, DEFAULTS, _jquery2.default.isPlainObject(options) && options);
        }
      }]);

      return AsScroll;
    }();

    var info = {
      version: '0.1.1'
    };

    var NAMESPACE = 'asScroll';
    var OtherAsScroll = _jquery2.default.fn.asScroll;

    var jQueryAsScroll = function jQueryAsScroll(options) {
      var _this = this;

      for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      if (typeof options === 'string') {
        var _ret3 = function() {
          var method = options;

          if (/^_/.test(method)) {

            return {
              v: false
            };
          } else if (/^(get)/.test(method)) {
            var instance = _this.first().data(NAMESPACE);

            if (instance && typeof instance[method] === 'function') {

              return {
                v: instance[method].apply(instance, args)
              };
            }
          } else {

            return {
              v: _this.each(

                function() {
                  var instance = _jquery2.default.data(this, NAMESPACE);

                  if (instance && typeof instance[method] === 'function') {
                    instance[method].apply(instance, args);
                  }
                }
              )
            };
          }
        }();

        if ((typeof _ret3 === 'undefined' ? 'undefined' : _typeof(_ret3)) === "object")

          return _ret3.v;
      }

      return this.each(

        function() {
          if (!(0, _jquery2.default)(this).data(NAMESPACE)) {
            (0, _jquery2.default)(this).data(NAMESPACE, new AsScroll(this, options));
          }
        }
      );
    };

    _jquery2.default.fn.asScroll = jQueryAsScroll;

    _jquery2.default.asScroll = _jquery2.default.extend({
      setDefaults: AsScroll.setDefaults,
      registerEasing: AsScroll.registerEasing,
      getEasing: AsScroll.getEasing,
      noConflict: function noConflict() {
        _jquery2.default.fn.asScroll = OtherAsScroll;

        return jQueryAsScroll;
      }
    }, info);
  }
);